/*
 * Copyright (c) 2001, Aslak Hellesy, BEKK Consulting
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * - Neither the name of BEKK Consulting nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
package middlegen.plugins.entitybean;

import middlegen.FileProducer;

/**
 * This plugin generates Enterprise Java Beans of type EntityBean conforming to
 * the EJB 2.0 specification, using Container-managed persistence (CMP).
 *
 * @author <a href="mailto:aslak.hellesoy@bekk.no">Aslak Hellesy</a>
 * @created 10. april 2002
 * @middlegen.plugin name="cmp20"
 */
public class CMP20Plugin extends Entity20Plugin {

   /**
    * @todo-javadoc Describe the field
    */
   private boolean _isFkcmp = true;

   /**
    * @todo-javadoc Describe the field
    */
   private static org.apache.log4j.Category _log = org.apache.log4j.Category.getInstance(CMP20Plugin.class.getName());


   /**
    * Describe what the CMP20Plugin constructor does
    *
    * @todo-javadoc Write javadocs for method parameter
    * @todo-javadoc Write javadocs for constructor
    * @todo-javadoc Write javadocs for method parameter
    */
   public CMP20Plugin() {
      super();
      setDisplayName("CMP 2.0");
   }


   /**
    * Sets the Fkcmp attribute of the Entity20Plugin object
    *
    * @todo defer warning until we see such a case.
    * @param flag The new Fkcmp value
    */
   public void setFkcmp(boolean flag) {
      _isFkcmp = flag;
      if (!_isFkcmp) {
         _log.warn("WARNING: When fkcmp=\"false\", you will not be able to use compound primary " +
               "keys where some of the columns are also foreign keys. This is because all columns of " +
               "a primary key must be CMP fields. See EJB 2.0. spec section 10.8.2"
               );
      }
   }


   /**
    * Gets the Fkcmp attribute of the Entity20Plugin object
    *
    * @return The Fkcmp value
    */
   public boolean isFkcmp() {
      return _isFkcmp;
   }


   /**
    * Describe what the method does
    *
    * @todo-javadoc Write javadocs for method
    */
   protected void registerFileProducers() {
      super.registerFileProducers();
      addConfiguredFileproducer(new FileProducer(getDestinationDir(), "{0}" + getBeansuffix() + ".java", getClass().getResource("entity-cmp-20.vm")));
   }
}
