/*
 * Copyright (c) 2001, Aslak Hellesy, BEKK Consulting
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * - Neither the name of BEKK Consulting nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
package middlegen.plugins.entitybean;

import java.util.Iterator;
import java.util.Collection;
import java.text.MessageFormat;
import middlegen.Table;
import middlegen.Util;
import middlegen.javax.JavaTable;
import middlegen.util.BooleanNode;

/**
 * Baseclass for Column decorators. Subclasses can add additional functionality.
 *
 * @author <a href="mailto:aslak.hellesoy@bekk.no">Aslak Hellesy</a>
 * @created 3. oktober 2001
 * @version $Id: Entity11Table.java,v 1.1 2004/08/20 06:53:43 manu Exp $
 */
public class Entity11Table extends JavaTable {

   /**
    * @todo-javadoc Describe the column
    */
   private BooleanNode _isRemote;

   /**
    * @todo-javadoc Describe the field
    */
   private boolean _generatePkClass;

   /**
    * Get static reference to Log4J Logger
    */
   private static org.apache.log4j.Category _log = org.apache.log4j.Category.getInstance(Entity11Table.class.getName());


   /**
    * Describe what the Entity11Table constructor does
    *
    * @param subject Describe what the parameter does
    * @todo-javadoc Write javadocs for method parameter
    * @todo-javadoc Write javadocs for constructor
    * @todo-javadoc Write javadocs for method parameter
    */
   public Entity11Table(Table subject) {
      super(subject);
   }


   /**
    * Describe what the setRemote constructor does
    *
    * @param flag Describe what the parameter does
    * @todo-javadoc Write javadocs for constructor
    * @todo-javadoc Write javadocs for method parameter
    */
   public void setRemote(boolean flag) {
      _isRemote.setValue(flag);
   }


   /**
    * Sets the BeanName attribute of the Entity11Table object
    *
    * @param s The new BeanName value
    */
   public void setBeanName(String s) {
      setBaseClassName(s);
   }


   /**
    * Sets the GeneratePk attribute of the Entity11Table object
    *
    * @param generatePkClass The new GeneratePk value
    */
   public void setGeneratePkClass(boolean generatePkClass) {
      setPrefsValue("generate-pk-class", Util.string(generatePkClass));
      _generatePkClass = generatePkClass;
   }


   /**
    * Gets the FinderColumns attribute of the Entity11Table object
    *
    * @return The FinderColumns value
    */
   public Collection getFinderColumns() {
      Entity11Plugin plugin = (Entity11Plugin)getPlugin();
      return getColumns(plugin.getFindersPredicate());
   }


   /**
    * Returns true if none of the set methods are exposed via the remote
    * interface
    *
    * @return The ReadOnly value
    */
   public boolean isReadOnly() {
      boolean result = true;
      for (Iterator columns = getColumns().iterator(); columns.hasNext(); ) {
         Entity11Column column = (Entity11Column)columns.next();
         if (column.isRemoteSet()) {
            // It isn't readonly. One column is exposed in a setter in remote
            result = false;
            break;
         }
      }
      return result;
   }


   /**
    * Returns true if a custom primary key class should be generated (by
    * XDoclet). If the pk consists of two columns or more, this always returns
    * true. If the pk is a single column, the user can choose (using
    * generatepk="true").
    *
    * @return The SimplePk value
    */
   public boolean isGeneratePkClass() {
      return _generatePkClass;
   }


   /**
    * Gets the Remote attribute of the Entity11Table object
    *
    * @return The Remote value
    */
   public boolean isRemote() {
      return _isRemote.isCompletelyTrue();
   }


   /**
    * Gets the Remote attribute of the Entity11Table object
    *
    * @return The Remote value
    */
   public BooleanNode getRemote() {
      return _isRemote;
   }



   /**
    * Gets the BeanName attribute of the EntityCMPClassType object
    *
    * @return The BeanName value
    * @todo-javadoc Write javadocs for method parameter
    */
   public String getBeanName() {
      return getBaseClassName();
   }



   /**
    * Gets the RemoteClassName attribute of the EntityCMPClassType object
    *
    * @return The RemoteClassName value
    * @todo-javadoc Write javadocs for method parameter
    */
   public String getRemoteClassName() {
      return getQualifiedInterfaceBaseClassName() + ((Entity11Plugin)getPlugin()).getRemotesuffix();
   }


   /**
    * @return The RemoteHomeClassName value
    * @todo-javadoc Write javadocs for method parameter
    */
   public String getRemoteHomeClassName() {
      return getQualifiedInterfaceBaseClassName() + ((Entity11Plugin)getPlugin()).getRemotehomesuffix();
   }


   /**
    * Gets the DataClassName attribute of the EntityCMPClassType object
    *
    * @return The DataClassName value
    * @todo-javadoc Write javadocs for method parameter
    */
   public String getDataClassName() {
      return getQualifiedInterfaceBaseClassName() + ((Entity11Plugin)getPlugin()).getDatasuffix();
   }


   /**
    * Gets the UtilClassName attribute of the Entity11Table object
    *
    * @return The UtilClassName value
    */
   public String getUtilClassName() {
      return getQualifiedInterfaceBaseClassName() + ((Entity11Plugin)getPlugin()).getUtilsuffix();
   }


   /**
    * Gets the DestinationClassName attribute of the EntityCMPClassType object
    *
    * @return The DestinationClassName value
    * @todo-javadoc Write javadocs for method parameter
    */
   public String getDestinationClassName() {
      return getBaseClassName() + ((Entity11Plugin)getPlugin()).getBeansuffix();
   }


   /**
    * Gets the InterfacePackage attribute of the Entity11Table object
    *
    * @return The InterfacePackage value
    */
   public String getInterfacePackage() {
      String interfacePackage = ((Entity11Plugin)getPlugin()).getInterfacePackage();
      interfacePackage = MessageFormat.format(interfacePackage, new String[]{getSqlName().toLowerCase()});
      return interfacePackage;
   }


   /**
    * Gets the QualifiedInterfaceBaseClassName attribute of the Entity11Table
    * object
    *
    * @return The QualifiedInterfaceBaseClassName value
    */
   public String getQualifiedInterfaceBaseClassName() {
      return Util.getQualifiedClassName(getInterfacePackage(), getBaseClassName());
   }


   /**
    * Gets the PkJavaType attribute of the Table object
    *
    * @todo refactor out dependency on EJB stuff.
    * @return The PkJavaType value
    * @todo-javadoc Write javadocs for method parameter
    */
   public String getPkClassName() {
      String result = null;
      if (isGeneratePkClass()) {
         result = getGeneratedPkClassName();
      }
      else {
         result = getSimplePkClassName();
      }
      return result;
   }


   /**
    * Gets the GeneratedPkClassName attribute of the Entity11Plugin object
    *
    * @return The GeneratedPkClassName value
    * @todo-javadoc Write javadocs for method parameter
    */
   public String getGeneratedPkClassName() {
      return getQualifiedInterfaceBaseClassName() + ((Entity11Plugin)getPlugin()).getPksuffix();
   }


   /**
    * Gets the RemoteJNDIName attribute of the Entity11Table object
    *
    * @return The RemoteJNDIName value
    */
   public String getRemoteJNDIName() {
      String result = null;
      Entity11Plugin plugin = (Entity11Plugin)getPlugin();
      if (plugin.getJNDIPrefix() != null) {
         result = plugin.getJNDIPrefix() + "/" + getBaseClassName() + plugin.getRemotehomesuffix();
      }
      else {
         result = getRemoteHomeClassName().replace('.', '/');
      }
      return result;
   }



   /**
    * Describe what the method does
    *
    * @todo-javadoc Write javadocs for method
    */
   protected void init() {
      super.init();
      // create boolean nodes for set/get that are parents of our table.
      // we initialise them to false if we're pk or fk
      _isRemote = ((Entity11Plugin)getPlugin()).getRemote().createChild(false);

      String isGeneratePkString = getPrefsValue("generate-pk-class");
      if (isGeneratePkString != null) {
         boolean isGeneratePk = Util.bool(isGeneratePkString);
         setGeneratePkClass(isGeneratePk);
         _log.debug("generate-pk-class from prefs (" + getSqlName() + "):" + isGeneratePkClass());
      }
      else {
         if (((Entity11Plugin)getPlugin()).isPkClass()) {
            // specified in Ant
            setGeneratePkClass(true);
            _log.debug("generate-pk-class from Ant (" + getSqlName() + "):" + isGeneratePkClass());
         }
         else {
            // not specified anywhere
            setGeneratePkClass(!isSimplePk());
            _log.debug("generate-pk-class from logic (" + getSqlName() + "):" + isGeneratePkClass());
         }
      }
   }
}

