/*
 * Copyright (c) 2001, Aslak Hellesy, BEKK Consulting
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * - Neither the name of BEKK Consulting nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
package middlegen.plugins.entitybean;

import java.util.Collection;
import java.util.Iterator;
import java.util.Observable;

import javax.swing.border.EtchedBorder;

import middlegen.Column;
import middlegen.ColumnMap;
import middlegen.CrossrefElement;
import middlegen.RelationshipRole;
import middlegen.Util;
import middlegen.util.BooleanNode;

/**
 * Represents a column in EJB 2.0 Entity Beans
 *
 * @author <a href="mailto:aslak.hellesoy@bekk.no">Aslak Hellesy </a>
 * @created 3. oktober 2001
 * @version $Id: Entity20Column.java,v 1.2 2004/08/23 13:08:56 manu Exp $
 */
public class Entity20Column extends Entity11Column {
   /**
    * @todo-javadoc Describe the column
    */
   private BooleanNode _isLocalSet;

   /**
    * @todo-javadoc Describe the field
    */
   private BooleanNode _isCreatable;

   /**
    * @todo-javadoc Describe the field
    */
   private BooleanNode _isUpdatable;

   /**
    * @todo-javadoc Describe the field
    */
   private BooleanNode _isCtor;

   /**
    * @todo-javadoc Describe the field
    */
   private BooleanNode _isLocalGet;

   /**
    * @todo-javadoc Describe the field
    */
   private String _porteeVO;

   /**
    * @todo-javadoc Describe the field
    */
   final static String BOTH = "both";

   /**
    * @todo-javadoc Describe the field
    */
   final static String LOCAL = "local";

   /**
    * @todo-javadoc Describe the field
    */
   final static String REMOTE = "remote";

   /**
    * @todo-javadoc Describe the field
    */
   final static String UNDEFINED = "undefined";

   /**
    * Get static reference to Log4J Logger
    */
   private static org.apache.log4j.Category _log = org.apache.log4j.Category.getInstance(Entity20Column.class.getName());


   /**
    * Describe what the EJB10Column constructor does
    *
    * @param subject Describe what the parameter does
    * @todo-javadoc Write javadocs for method parameter
    * @todo-javadoc Write javadocs for constructor
    * @todo-javadoc Write javadocs for method parameter
    */
   public Entity20Column(Column subject) {
      super(subject);
      //		System.out.println("JavaColumn() subject = " + subject.getSqlName());
   }


   /**
    * Describe what the setLocalSet constructor does
    *
    * @param flag Describe what the parameter does
    * @todo-javadoc Write javadocs for constructor
    * @todo-javadoc Write javadocs for method parameter
    */
   public void setLocalSet(boolean flag) {
      //System.out.println("update local-set");
      setPrefsValue("local-set", Util.string(flag));
      // only set if value changed (to avoid infinite observer loop)
      if (flag != _isLocalSet.isCompletelyTrue()) {
         _isLocalSet.setValue(flag);
      }
   }


   /**
    * Describe what the setLocalGet constructor does
    *
    * @param flag Describe what the parameter does
    * @todo-javadoc Write javadocs for constructor
    * @todo-javadoc Write javadocs for method parameter
    */
   public void setLocalGet(boolean flag) {
      setPrefsValue("local-get", Util.string(flag));
      // only set if value changed (to avoid infinite observer loop)
      if (flag != _isLocalGet.isCompletelyTrue()) {
         _isLocalGet.setValue(flag);
      }
   }


   /**
    * Sets the Creatable attribute of the Entity20Column object
    *
    * @param flag The new Creatable value
    */
   public void setCreatable(boolean flag) {
      _log.debug("update creatable passe  : " + flag);

      setPrefsValue("ctorVO", Util.string(flag));
      // only set if value changed (to avoid infinite observer loop)
      if (flag != _isCreatable.isCompletelyTrue()) {
         _isCreatable.setValue(flag);
      }
   }


   /**
    * Sets the Updatable attribute of the Entity20Column object
    *
    * @param flag The new Updatable value
    */
   public void setUpdatable(boolean flag) {
      _log.debug("update updatable set : " + flag);

      setPrefsValue("updatable", Util.string(flag));
      // only set if value changed (to avoid infinite observer loop)
      if (flag != _isUpdatable.isCompletelyTrue()) {
         _isUpdatable.setValue(flag);
      }
   }


   /**
    * Sets the PorteeVO attribute of the Entity20Column object
    *
    * @param porteeVO The new PorteeVO value
    */
   public void setPorteeVO(String porteeVO) {
      _log.debug(" set porteeVO = " + porteeVO);
      this._porteeVO = porteeVO;
      setPrefsValue("porteeVO", porteeVO);

   }


   /**
    * Sets the Ctor attribute of the Entity20Column object
    *
    * @param flag The new Ctor value
    */
   public void setCtor(boolean flag) {
      _log.debug("setCTOR to : " + flag);

      setPrefsValue("ctorVO", Util.string(flag));
      // only set if value changed (to avoid infinite observer loop)
      if (flag != _isCtor.isCompletelyTrue()) {
         _isCtor.setValue(flag);
      }
   }


   /**
    * Gets the SetterViewType attribute of the Entity20Column object
    *
    * @return The SetterViewType value
    */
   public String getSetterViewType() {
      return viewType(isLocalSet(), isRemoteSet());
   }


   /**
    * Gets the GetterViewType attribute of the Entity20Column object
    *
    * @return The GetterViewType value
    */
   public String getGetterViewType() {
      return viewType(isLocalGet(), isRemoteGet());
   }


   /**
    * Gets the GetterLocalOrRemote attribute of the Entity20Column object
    *
    * @return The GetterLocalOrRemote value
    */
   public boolean isGetterLocalOrRemote() {
      return (isLocalGet() || isRemoteGet());
   }


   /**
    * Gets the SetterLocalOrRemote attribute of the Entity20Column object
    *
    * @return The SetterLocalOrRemote value
    */
   public boolean isSetterLocalOrRemote() {
      return (isLocalSet() || isRemoteSet());
   }


   /**
    * Gets the RemoteGet attribute of the Entity11Column object
    *
    * @return The RemoteGet value
    */
   public BooleanNode getLocalGet() {
      return _isLocalGet;
   }


   /**
    * Gets the RemoteSet attribute of the Entity11Column object
    *
    * @return The RemoteSet value
    */
   public BooleanNode getLocalSet() {
      return _isLocalSet;
   }


   /**
    * Gets the LocalSet attribute of the Entity20Column object
    *
    * @return The LocalSet value
    */
   public boolean isLocalSet() {
      return getLocalSet().isCompletelyTrue();
   }


   /**
    * Gets the LocalGet attribute of the Entity20Column object
    *
    * @return The LocalGet value
    */
   public boolean isLocalGet() {
      boolean result = getLocalGet().isCompletelyTrue();
      return result;
   }


   /**
    * @return
    */
   public boolean isCtor() {

      boolean result = getCtor().isCompletelyTrue();
      return result;
   }


   /**
    * Gets the Creatable attribute of the Entity20Column object
    *
    * @return The Creatable value
    */
   public boolean isCreatable() {

      boolean result = getCreatable().isCompletelyTrue();
      return result;
   }


   /**
    * Gets the Creatable attribute of the Entity20Column object
    *
    * @return The Creatable value
    */
   public BooleanNode getCreatable() {
      return _isCreatable;
   }


   /**
    * Gets the Updatable attribute of the Entity20Column object
    *
    * @return The Updatable value
    */
   public boolean isUpdatable() {

      boolean result = getUpdatable().isCompletelyTrue();
      return result;
   }


   /**
    * Gets the Updatable attribute of the Entity20Column object
    *
    * @return The Updatable value
    */
   public BooleanNode getUpdatable() {
      return _isUpdatable;
   }


   /**
    * Gets the PorteeVO attribute of the Entity20Column object
    *
    * @return The PorteeVO value
    */
   public String getPorteeVO() {
      _log.debug(" porteeVO = " + this._porteeVO);
      return this._porteeVO;
   }


   /**
    * Gets the Ctor attribute of the Entity20Column object
    *
    * @return The Ctor value
    */
   public BooleanNode getCtor() {
      return _isCtor;
   }


   /**
    * Gets the Fk attribute of the Entity20Column object
    *
    * @return The Fk value
    */
   public String getFk() {
      boolean match = false;
      String s = "";
      if (isFk()) {

         //
         _log.debug(getSqlName() + " of " + getTable().getName()
                + " is a Fk in ?");
         Collection roles = getTable().getRelationshipRoles();
         Iterator it = roles.iterator();
         while (it.hasNext()) {
            RelationshipRole role = (RelationshipRole)it.next();

            _log.debug(" role " + role.getName() + " "
                   + role.getOrigin().getName() + " "
                   + role.getRelation().getLeftTable().getName() + " "
                   + role.getTarget().getName() + " TargetMany ? "
                   + role.getRelation().getRightTable().getName() + " "
                   + role.isTargetMany() + " role enabled "
                   + role.isEnabled());

            if (!role.isEnabled() || role.isTargetMany()) {
               continue;
            }

            if (role.getRelation().getLeftRole() != null) {
               _log.debug(" size leftrole"
                      + role.getRelation().getLeftRole()
                     .getColumnMaps().length);
               for (int i = 0; i < role.getRelation().getLeftRole()
                     .getColumnMaps().length; i++) {
                  _log.debug(" CTOR Left pk/fk : "
                         + role.getRelation().getLeftRole()
                        .getColumnMaps()[i].getPrimaryKey()
                         + "/"
                         + role.getRelation().getLeftRole()
                        .getColumnMaps()[i].getForeignKey());
                  if (role.getRelation().getLeftRole().getColumnMaps()[i]
                        .getForeignKey().equalsIgnoreCase(getSqlName())) {
                     //MANU
                     match = true;
                  }

               }

            }

            if (role.getRelation().getRightRole() != null) {
               _log.debug(" size Rightrole"
                      + role.getRelation().getRightRole()
                     .getColumnMaps().length);
               for (int i = 0; i < role.getRelation().getRightRole()
                     .getColumnMaps().length; i++) {
                  _log.debug(" CTOR pk/fk : "
                         + role.getRelation().getRightRole()
                        .getColumnMaps()[i].getPrimaryKey()
                         + "/"
                         + role.getRelation().getRightRole()
                        .getColumnMaps()[i].getForeignKey());
               }

            }
            s = role.getTarget().getName().toLowerCase();
            s = Util.capitalise(s);
            s = Util.singularise(s);
            if (match) {
               _log.debug(getSqlName()
                      + " est une fk crer meth value object avec Table : "
                      + s);

            }

         }
      }

      return match == true ? s : "";
   }


   /**
    * Describe what the method does
    *
    * @param o Describe what the parameter does
    * @param arg Describe what the parameter does
    * @todo-javadoc Write javadocs for method
    * @todo-javadoc Write javadocs for method parameter
    * @todo-javadoc Write javadocs for method parameter
    */
   public void update(Observable o, Object arg) {

      super.update(o, arg);

      _log.debug("objet observ : " + o + o.getClass().getName());

      if (o == _isLocalSet) {
         setLocalSet(_isLocalSet.isCompletelyTrue());
      }
      else if (o == _isLocalGet) {
         setLocalGet(_isLocalGet.isCompletelyTrue());
      }
      else if (o == _isCreatable) {
         setCreatable(_isCreatable.isCompletelyTrue());
      }
      else if (o == _isUpdatable) {
         setUpdatable(_isUpdatable.isCompletelyTrue());
      }
      else if (o == _isCtor) {
         setCtor(_isCtor.isCompletelyTrue());
      }
   }


   /**
    * Overrides method in superclass. returns true if viewtype = "both" or
    * "remote"
    *
    * @return The RemoteEnabled value
    */
   protected boolean isRemoteEnabled() {
      String viewtype = ((Entity20Plugin)getPlugin()).getViewtype();
      return viewtype.equals("both") || viewtype.equals("remote");
   }


   /**
    * Gets the LocalEnabled attribute of the Entity20Column object
    *
    * @return The LocalEnabled value
    */
   protected boolean isLocalEnabled() {
      String viewtype = ((Entity20Plugin)getPlugin()).getViewtype();
      return viewtype.equals("both") || viewtype.equals("local");
   }


   /**
    * Describe what the method does
    *
    * @todo-javadoc Write javadocs for method
    */
   protected void init() {

      super.init();

      Entity20Table table = (Entity20Table)getTable();

      _log.debug(" Entity20Col " + this.getSqlName());

      // these falses will be overriden...
      _isLocalSet = table.getLocal().createChild(false);
      _isLocalGet = table.getLocal().createChild(false);
      _isCreatable = table.getLocal().createChild(false);
      _isUpdatable = table.getLocal().createChild(false);
      _isCtor = table.getLocal().createChild(false);

      // We want to know when the view-type changes
      //		_isLocalSet.addObserver(this);
      _isLocalGet.addObserver(this);
      _isUpdatable.addObserver(this);
      _isCreatable.addObserver(this);
      _isCtor.addObserver(this);

      // here...

      // local
      // Don't enable setters for pk fields or fk fields. They are read-only.
      boolean isKey = isPk() || isFk();
      boolean computedLocalSet = !isKey && isLocalEnabled()
             && !isReadonlyDefault();
      if (getPrefsValue("local-set") != null) {
         boolean prefsLocalSet = Util.bool(getPrefsValue("local-set"));
         if (computedLocalSet == false && prefsLocalSet == true) {
            _log.warn("WARNING: The "
                   + getPlugin().getName()
                   + " is configured with readonly=\"true\", "
                   + "but the prefs settings for the column "
                   + table.getSqlName()
                   + "."
                   + getSqlName()
                   + " indicates that a local setter for this column should be generated. "
                   + "Middlegen will use the prefs setting, so the generated "
                   + table.getLocalClassName()
                   + "'s set"
                   + getCapitalisedVariableName()
                   + " method "
                   + "will exist, causing the bean NOT to be read only.");
         }
         setLocalSet(prefsLocalSet);
      }
      else {
         setLocalSet(computedLocalSet);
      }

      // Getter
      boolean isFkAndNotPk = isFk() && !isPk();
      if (getPrefsValue("local-get") != null) {
         setLocalGet(Util.bool(getPrefsValue("local-get")));
      }
      else {
         // Only enable getters for fields that are pk or regular (i.e. not
         // fk)
         if (isFk()) {
            if (isPk()) {
               setLocalGet(isLocalEnabled());
               isFkAndNotPk = false;
            }
            else {
               setLocalGet(false);
            }
         }
         else {
            setLocalGet(isLocalEnabled());
            isFkAndNotPk = false;
         }
      }

      // warn if viewtype in prefs differs from that of the plugin
      Entity20Plugin plugin = (Entity20Plugin)getPlugin();
      boolean agreedOnSetter = getSetterViewType().equals(
            plugin.getViewtype());
      boolean agreedOnGetter = getGetterViewType().equals(
            plugin.getViewtype());
      String setter = getSetterName() + "[" + getSetterViewType() + "]";
      String getter = getGetterName() + "[" + getGetterViewType() + "]";
      String methodNames = null;
      String has = "has";
      if (!agreedOnSetter && !agreedOnGetter) {
         methodNames = setter + " and " + getter;
         has = "both have";
      }
      else if (!agreedOnSetter) {
         methodNames = setter;
      }
      else {
         methodNames = getter;
      }

      if (!(isPk() || isFk())
             && ((!agreedOnSetter && !isReadonlyDefault()) || !agreedOnGetter)) {
         _log.warn("WARNING ("
                + plugin.getName()
                + "): According to the prefs file, "
                + table.getBeanName()
                + "'s "
                + methodNames
                + " "
                + has
                + " a viewtype that doesn't match that "
                + "of the plugin. The plugin has specified viewtype=\""
                + plugin.getViewtype()
                + "\". "
                + "Middlegen will ignore the plugin setting and use the viewtype "
                + "stored in the prefs file. If your intention was "
                + "to use the "
                + plugin.getViewtype()
                + " viewtype, please edit your prefs file "
                + "or use the gui to set a matching view type. Otherwise you can ignore this warning.");
      }

      //		if (getPrefsValue("updatable") != null) {
      //			_log.debug(" init col updatble with : "
      //					+ getPrefsValue("updatable"));
      //			setUpdatable(Util.bool(getPrefsValue("updatable")));
      //		}
      //
      //		if (getPrefsValue("ctorVO") != null) {
      //			_log.debug(" init col creatable with : "
      //					+ getPrefsValue("ctorVO"));
      //			setCreatable(Util.bool(getPrefsValue("ctorVO")));
      //		}

      if (getPrefsValue("porteeVO") != null) {
         _log.debug(" init col porteeVO with : "
                + getPrefsValue("porteeVO"));
         setPorteeVO(getPrefsValue("porteeVO"));
      }

      _log.debug("avant init ctorVO from prefs pour champ "
             + this.getSqlName() + " ... lu depuis prefs"
             + getPrefsValue("ctorVO"));
      if (getPrefsValue("ctorVO") != null) {
         _log.debug(" init col ctorVO with : " + getPrefsValue("ctorVO"));
         setCtor(Util.bool(getPrefsValue("ctorVO")));
      }

      // some debug code

   }


   /**
    * Returns "local", "remote" or "both". Used by template to insert correct
    * XDoclet tag value for view-type
    *
    * @param isLocal Describe what the parameter does
    * @param isRemote Describe what the parameter does
    * @return Describe the return value
    * @todo-javadoc Write javadocs for method
    * @todo-javadoc Write javadocs for method parameter
    * @todo-javadoc Write javadocs for method parameter
    * @todo-javadoc Write javadocs for return value
    */
   static String viewType(boolean isLocal, boolean isRemote) {
      if (isRemote) {
         if (isLocal) {
            return BOTH;
         }
         else {
            return REMOTE;
         }
      }
      else if (isLocal) {
         return LOCAL;
      }
      return UNDEFINED;
   }

}
